-- ToDi_Attachment
--
-- Version 1.0
-- Autor Ralf08
-- 
-- Change log:
-- 27.01.2019 / Ralf08 / first release -> replacment for LS17 FrontloaderDisplay



--local FirstRun
ToDi_Attachment = {};


local ToDi_ang = 0

function ToDi_Attachment.prerequisitesPresent(specializations)
	--print("ToDi_Attachment.prerequisitesPresent(specializations)")
	return true
end;

function ToDi_Attachment.registerEventListeners(vehicleType)
	--print("ToDi_Attachment.registerEventListeners(vehicleType)")
	for i, v in ipairs({"onLoad","onPreAttach","onPreDetach","onUpdate","onRegisterActionEvents"}) do
		SpecializationUtil.registerEventListener(vehicleType, v, ToDi_Attachment);
	end;
end;


function ToDi_Attachment:onLoad()
	--print("ToDi_Attachment:onLoad()")
	--print(self.typeName)
	self.ToDi = {}
	self.ToDi.corAngle = 0
	self.ToDi.corHeight = 0
	self.ToDi.latch = false
	self.ToDi.parentVehicle = self
	self.ToDi.xL = 0
	self.ToDi.yL = 0
	self.ToDi.zL = 0
	self.ToDi.dir = 1
	
	if self.components ~= nil then
		self.ToDi.node = self.components[1].node
	end;
		

	--Dischargeable
	if SpecializationUtil.hasSpecialization(Dischargeable, self.specializations) then
		--print("ToDi DISCHARGEABLE")
		if self.spec_dischargeable.currentDischargeNode.node ~= nil then
			--print("ToDi currentDischargeNode")
			self.ToDi.node = self.spec_shovel.shovelDischargeInfo.node
		end;
	end;
	

	--stumpCutter
	if SpecializationUtil.hasSpecialization(StumpCutter, self.specializations) then
		--print("ToDi stumpCutter")
		if self.i3dMappings.cutNdeRefFrame ~= nil then
			--print("ToDi cutNdeRefFrame")
			self.ToDi.node = I3DUtil.indexToObject(self.components,self.i3dMappings.cutNdeRefFrame)
		elseif self.i3dMappings.cutNodeRefFrame ~= nil then
			--print("ToDi cutNodeRefFrame")
			self.ToDi.node = I3DUtil.indexToObject(self.components,self.i3dMappings.cutNodeRefFrame)
		elseif self.i3dMappings.cutNode ~= nil then
			--print("ToDi cutNode")
			self.ToDi.node = I3DUtil.indexToObject(self.components,self.i3dMappings.cutNode)
		end;
	end;

	--TreeSaw
	if SpecializationUtil.hasSpecialization(TreeSaw, self.specializations) then
		--print("ToDi TREESAW")
		if self.spec_treeSaw.cutNode ~= nil then
			--print("ToDi cutNode")
			self.ToDi.node = self.spec_treeSaw.cutNode
		end;
	end;
	
	local __,yX,__ = localDirectionToLocal(self.ToDi.node,self.components[1].node, 1, 0, 0)
	local __,yY,__ = localDirectionToLocal(self.ToDi.node,self.components[1].node, 0, 1, 0)
	local __,yZ,__ = localDirectionToLocal(self.ToDi.node,self.components[1].node, 0, 0, 1)
	
	yX = math.abs(yX) - 1
	yY = math.abs(yY) - 1
	yZ = math.abs(yZ) - 1
	
	if yX > yY and yX > yZ then
		--print("xL")
		self.ToDi.xL = 1
	elseif yY > yX and yY > yZ then
		--print("yL")
		self.ToDi.yL = 1
	elseif yZ > yX and yZ > yY then
		--print("zL")
		self.ToDi.zL = 1
	else
		--print("ToDi Error, standard configuration")
		self.ToDi.zL = 1
	end;
end;

function ToDi_Attachment:onUpdate(dt, isActiveForInput)
	if isActiveForInput then
		-- --print("ToDi_Attachment:onUpdate() "..self.typeName.." "..dt) end;
		if self.ToDi.node ~= nil and self.ToDi.parentVehicle ~= nil then
			
			local x,y,z = localToLocal(self.ToDi.node,self.ToDi.parentVehicle.components[1].node, 0, 0, 0)
			local xD,yD,zD = localDirectionToLocal(self.ToDi.node,self.ToDi.parentVehicle.components[1].node, self.ToDi.xL, self.ToDi.yL, self.ToDi.zL)
						
			self.ToDi.parentVehicle.ToDi.ToDi_HoT = math.floor((y + self.ToDi.corHeight)*100+0.5)/100
			self.ToDi.parentVehicle.ToDi.ToDi_ang = self.ToDi.dir * (math.floor(math.deg(math.atan2(yD,zD)) + 0.5) - 90 + self.ToDi.corAngle)
		end;
		
		self.ToDi.parentVehicle.ToDi.show = true
		self.ToDi.latch = true
		
	elseif self.ToDi.latch then	
		self.ToDi.parentVehicle.ToDi.show = false
		self.ToDi.latch = false
	end;
end;

function ToDi_Attachment:onPreAttach(attacherVehicle, inputJointDescIndex, jointDescIndex)
	--print("ToDi_Attachment:onPostAttach()")
	self.ToDi.parentVehicle = self:getRootVehicle()
	local __,__,z = localToLocal(self.ToDi.node,self.ToDi.parentVehicle.components[1].node, 0, 0, 0)
	if z > 0 then
		self.ToDi.dir = 1
	else
		self.ToDi.dir = -1
	end;
end;

function ToDi_Attachment:onPreDetach()
	--print("ToDi_Attachment:onPreDetach()")
	if self.ToDi.parentVehicle.ToDi ~= nil then
		self.ToDi.parentVehicle.ToDi.show = false
	end;
	self.ToDi.latch = false
	self.ToDi.parentVehicle = nil
	self.ToDi.dir = 1
end;

function ToDi_Attachment:onRegisterActionEvents(isSelected, isOnActiveVehicle)
    --print("ToDi_Attachment:onRegisterActionEvents()")
    local __, eventNameH, eventNameL
    if isSelected then
		__, eventNameH = InputBinding.registerActionEvent(g_inputBinding, 'ToDi_SetHeight',self, ToDi_Attachment.ToDi_SetHeight ,false ,true ,false ,true)
		__, eventNameH = InputBinding.registerActionEvent(g_inputBinding, 'ToDi_SetAngle',self, ToDi_Attachment.ToDi_SetAngle ,false ,true ,false ,true)
		__, eventNameL = InputBinding.registerActionEvent(g_inputBinding, 'ToDi_PLUS',self, ToDi_Attachment.ToDi_PLUS ,false ,true ,false ,true)
		__, eventNameL = InputBinding.registerActionEvent(g_inputBinding, 'ToDi_MINUS',self, ToDi_Attachment.ToDi_MINUS ,false ,true ,false ,true)
		g_inputBinding.events[eventNameH].displayPriority = 1
		g_inputBinding.events[eventNameL].displayPriority = 5
    end
end;

function ToDi_Attachment:ToDi_SetHeight(actionName, keyStatus, arg4, arg5, arg6)
	--print("ToDi_Attachment:ToDi_SetHeight()")
 	self.ToDi.corHeight = self.ToDi.corHeight - self.ToDi.parentVehicle.ToDi.ToDi_HoT
end;

function ToDi_Attachment:ToDi_SetAngle(actionName, keyStatus, arg4, arg5, arg6)
	--print("ToDi_Attachment:ToDi_SetAngle()")
	self.ToDi.corAngle = self.ToDi.corAngle - (self.ToDi.dir*self.ToDi.parentVehicle.ToDi.ToDi_ang)
end;

function ToDi_Attachment:ToDi_PLUS(actionName, keyStatus, arg4, arg5, arg6)
	--print("ToDi_Attachment:ToDi_PLUS()")
	ToDi_Vehicle.ToDi_size = ToDi_Vehicle.ToDi_size * 1.1
end;

function ToDi_Attachment:ToDi_MINUS(actionName, keyStatus, arg4, arg5, arg6)
	--print("ToDi_Attachment:ToDi_MINUS()")
	ToDi_Vehicle.ToDi_size = ToDi_Vehicle.ToDi_size / 1.1
end;
